import msigi.netclient.*;
import msigi.loadconfig.*;
import msigi.netgpibc.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.lang.reflect.InvocationTargetException;

public class DMM implements ActionListener {

	// Configuration
	private LoadConfig lc;
	private boolean usePreValues = false;
	private String configHostname = "";
	private String configPort = "";
	private String configAddress = "";

	// State and network
	private String errorMessage;
	private String readField;

	// Frames and panels
	private JFrame mainFrame;
	private JPanel mainPanel;
	//private JPanel displayPanel;
	private JPanel typePanel;
	private JPanel rangePanel;
	private JPanel triggerPanel;
	private JPanel terminalsPanel;
	private JPanel hostRowPanel;
	private JPanel portAddressRowPanel;

	// Displays, buttons, and text entry fields
	private JTextField readDisplay;
	private JLabel hostEntryLabel;
	private JTextField hostEntry;
	private JLabel portEntryLabel;
	private JTextField portEntry;
	private JLabel addressEntryLabel;
	private JTextField addressEntry;
	private JButton startStopButton;

	// Placeholders
	private JLabel measureType;
	private JLabel measureRange;
	private JLabel measureTrigger;
	private JLabel measureTerminals;

	// Constructor
	public DMM() {
		loadAndStoreConfiguration();
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		} );
	}

	// Configuration
	private void loadAndStoreConfiguration() {
		String value;
		int intValue;
		lc = new LoadConfig("config.txt");
		try {
			lc.readFile();
			value = lc.getParamByName("hostname");
			if (value != null) {
				configHostname = value;
			}
			value = lc.getParamByName("port");
			if (value != null) {
				configPort = value;
			}
			value = lc.getParamByName("address");
			if (value != null) {
				configAddress = value;
			}
			usePreValues = true;
		} catch (FileNotFoundException ex) {
		} catch (IOException ex) {
		}
	}

	// Create and show GUI
	private void createAndShowGUI() {

		// Variables
		GridBagConstraints c;
		GridBagConstraints d;
		Insets insets = new Insets(3, 3, 3, 3);
		int b = 3;

		// Frames and panels
		JFrame.setDefaultLookAndFeelDecorated(true);
		mainFrame = new JFrame("MSIGI Network DMM");
		mainPanel = new JPanel();
		mainPanel.setLayout(new GridBagLayout());
		//displayPanel = new JPanel();
		//displayPanel.setLayout(new GridLayout(1,0));
		//displayPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		typePanel = new JPanel();
		typePanel.setLayout(new GridBagLayout());
		rangePanel = new JPanel();
		rangePanel.setLayout(new GridBagLayout());
		triggerPanel = new JPanel();
		triggerPanel.setLayout(new GridBagLayout());
		terminalsPanel = new JPanel();
		terminalsPanel.setLayout(new GridBagLayout());
		hostRowPanel = new JPanel();
		hostRowPanel.setLayout(new GridBagLayout());
		hostRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		portAddressRowPanel = new JPanel();
		portAddressRowPanel.setLayout(new GridBagLayout());
		portAddressRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));

		// Display
		c = new GridBagConstraints();
		readDisplay = new JTextField(8);
		readDisplay.setFont(new Font("SansSerif", Font.PLAIN, 48));
		readDisplay.setBackground(Color.black);
		readDisplay.setForeground(Color.green);
		readDisplay.setHorizontalAlignment(JTextField.CENTER);
		//readDisplay.setEditable(false);
		readDisplay.setText("3.14159 V");
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 4;
		c.weightx = 0.5;
		c.weighty = 0.5;
		c.insets = insets;
		c.fill = GridBagConstraints.BOTH;
		mainPanel.add(readDisplay, c);

		// DMM controls
		d = new GridBagConstraints();
		measureType = new JLabel("Type");
		d.fill = GridBagConstraints.BOTH;
		d.weightx = 0.5;
		typePanel.add(measureType, d);

		measureRange = new JLabel("Range");
		rangePanel.add(measureRange, d);

		measureTrigger = new JLabel("Trigger");
		triggerPanel.add(measureTrigger, d);

		measureTerminals = new JLabel("Terminals");
		terminalsPanel.add(measureTerminals, d);


		c = new GridBagConstraints();
		c.weightx = 0.5;
		c.weighty = 0.5;
		// Host row
		hostEntryLabel = new JLabel("Hostname: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.gridy = 0;
		c.ipadx = 10;
		c.ipady = 5;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0;
		c.weighty = 0.5;
		hostRowPanel.add(hostEntryLabel, c);

		hostEntry = new JTextField(15);
		hostEntry.setText(configHostname);
		c.gridx = 1;
		c.weightx = 1;
		hostRowPanel.add(hostEntry, c);

		// Port and address row
		portEntryLabel = new JLabel("Port: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.weightx = 0;
		portAddressRowPanel.add(portEntryLabel, c);

		portEntry = new JTextField(10);
		portEntry.setText(configPort);
		c.gridx = 1;
		c.weightx = 0.5;
		portAddressRowPanel.add(portEntry, c);

		addressEntryLabel = new JLabel("GPIB Address: ", SwingConstants.CENTER);
		c.gridx = 2;
		c.weightx = 0;
		portAddressRowPanel.add(addressEntryLabel, c);

		addressEntry = new JTextField(10);
		addressEntry.setText(configAddress);
		c.gridx = 3;
		c.weightx = 0.5;
		portAddressRowPanel.add(addressEntry, c);

		// Start/stop button
		startStopButton = new JButton("Start"); 
		startStopButton.setMnemonic('s');

		// Set up panel hierarchy
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 1;
		c.gridheight = 4;
		c.weightx = 0.5;
		c.weighty = 0.5;
		mainPanel.add(typePanel, c);
		c.gridx = 1;
		mainPanel.add(rangePanel, c);
		c.gridx = 2;
		c.gridheight = 1;
		mainPanel.add(triggerPanel, c);
		c.gridx = 3;
		mainPanel.add(terminalsPanel, c);
		
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 2;
		c.gridy = 2;
		c.gridwidth = 2;
		c.weightx = 0.5;
		c.weighty = 0.5;
		mainPanel.add(hostRowPanel, c);
		c.gridy = 3;
		mainPanel.add(portAddressRowPanel, c);
		c.gridy = 4;
		c.insets = insets;
		mainPanel.add(startStopButton, c);

		// Show the gui
		mainFrame.getContentPane().add(mainPanel, BorderLayout.CENTER);
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.pack();
		mainFrame.setLocationRelativeTo(null);
		updateGUIState();
		startStopButton.requestFocus();
		mainFrame.setVisible(true);

	}

	// Event handler
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == startStopButton) {
			startStopButtonHandler();
		}
	}

	// Connect button handler
	private void startStopButtonHandler() {
	}

	// Update GUI state (enables and disables buttons)
	public void updateGUIState() {
	}	

	// Thread-safe way to call updateGUIState()
	public void updateGUIStateSafe() {
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				updateGUIState();
			}
		} );
	}

	// Thread-safe way to set read field
	private void setReadField() {
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				readDisplay.setText(readField);
			}
		} );
	}

	// Show an error message
	public void popupErrorMessage(String message) {
		errorMessage = message;
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(mainFrame,errorMessage,
					"Error",JOptionPane.ERROR_MESSAGE);
			}
		} );
	}

	// Main program
	public static void main(String[] args) {
		DMM dmm_gui;
		dmm_gui = new DMM();
	}

}
