import msigi.netclient.*;
import msigi.loadconfig.*;
import msigi.netgpibc.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.lang.reflect.InvocationTargetException;

public class GPIBInteractiveControl implements ActionListener {

	// Configuration
	private LoadConfig lc;
	private boolean usePreValues = false;
	private String configHostname = "";
	private String configPort = "";
	private String configAddress = "";
	private String configWrite = "";

	// State and network
	private String errorMessage;
	private String readField;
	private boolean isConnected = false;
	private boolean isAttemptingToConnect = false;
	private boolean isClosingConnection = false;
	private GPIBCommunicationThread gpibThread;
	private ConnectStatusHandler connectStatusHandler;
	private CloseStatusHandler closeStatusHandler;
	private WriteStatusHandler writeStatusHandler;
	private ReadStatusHandler readStatusHandler;
	private NetGPIBClient gpib;

	// Frames and panels
	private GridBagConstraints c;
	private JFrame mainFrame;
	private JPanel mainPanel;
	private JPanel hostRowPanel;
	private JPanel portAddressRowPanel;
	private JPanel connectionRowPanel;
	private JPanel writeRowPanel;
	private JPanel commRowPanel;
	private JPanel readRowPanel;

	// Displays, buttons, and text entry fields
	private JLabel hostEntryLabel;
	private JTextField hostEntry;
	private JLabel portEntryLabel;
	private JTextField portEntry;
	private JLabel addressEntryLabel;
	private JTextField addressEntry;
	private JButton connectButton;
	private JButton disconnectButton;
	private JLabel writeEntryLabel;
	private JTextField writeEntry;
	private JButton writeButton;
	private JButton readButton;
	private JButton queryButton;
	private JLabel readDisplayLabel;
	private JTextField readDisplay;

	// Constructor
	public GPIBInteractiveControl() {
		loadAndStoreConfiguration();
		gpibThread = new GPIBCommunicationThread();
		connectStatusHandler = new ConnectStatusHandler();
		closeStatusHandler = new CloseStatusHandler();
		writeStatusHandler = new WriteStatusHandler();
		readStatusHandler = new ReadStatusHandler();
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		} );
	}

	// Configuration
	private void loadAndStoreConfiguration() {
		String value;
		int intValue;
		lc = new LoadConfig("config.txt");
		try {
			lc.readFile();
			value = lc.getParamByName("hostname");
			if (value != null) {
				configHostname = value;
			}
			value = lc.getParamByName("port");
			if (value != null) {
				configPort = value;
			}
			value = lc.getParamByName("address");
			if (value != null) {
				configAddress = value;
			}
			value = lc.getParamByName("writestring");
			if (value != null) {
				configWrite = value;
			}
			usePreValues = true;
		} catch (FileNotFoundException ex) {
		} catch (IOException ex) {
		}
	}

	// Create and show GUI
	private void createAndShowGUI() {

		// Frames and panels
		int b = 3;
		JFrame.setDefaultLookAndFeelDecorated(true);
		mainFrame = new JFrame("MSIGI Network GPIB Interactive Control");
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS));
		hostRowPanel = new JPanel();
		hostRowPanel.setLayout(new GridBagLayout());
		hostRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		portAddressRowPanel = new JPanel();
		portAddressRowPanel.setLayout(new GridBagLayout());
		portAddressRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		connectionRowPanel = new JPanel();
		connectionRowPanel.setLayout(new GridBagLayout());
		connectionRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		writeRowPanel = new JPanel();
		writeRowPanel.setLayout(new GridBagLayout());
		writeRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		commRowPanel = new JPanel();
		commRowPanel.setLayout(new GridBagLayout());
		commRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));
		readRowPanel = new JPanel();
		readRowPanel.setLayout(new GridBagLayout());
		readRowPanel.setBorder(BorderFactory.createEmptyBorder(b,b,b,b));

		// Displays and controls
		c = new GridBagConstraints();

		// Host row
		hostEntryLabel = new JLabel("Hostname: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.gridy = 0;
		c.ipadx = 10;
		c.ipady = 5;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0;
		c.weighty = 0.5;
		hostRowPanel.add(hostEntryLabel, c);

		hostEntry = new JTextField(15);
		hostEntry.setText(configHostname);
		c.gridx = 1;
		c.weightx = 1;
		hostRowPanel.add(hostEntry, c);

		// Port and address row
		portEntryLabel = new JLabel("Port: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.weightx = 0;
		portAddressRowPanel.add(portEntryLabel, c);

		portEntry = new JTextField(10);
		portEntry.setText(configPort);
		c.gridx = 1;
		c.weightx = 0.5;
		portAddressRowPanel.add(portEntry, c);

		addressEntryLabel = new JLabel("GPIB Address: ", SwingConstants.CENTER);
		c.gridx = 2;
		c.weightx = 0;
		portAddressRowPanel.add(addressEntryLabel, c);

		addressEntry = new JTextField(10);
		addressEntry.setText(configAddress);
		c.gridx = 3;
		c.weightx = 0.5;
		portAddressRowPanel.add(addressEntry, c);

		// Connection row
		connectButton = new JButton("Connect");
		connectButton.addActionListener(this);
		connectButton.setMnemonic('c');
		c.gridx = 0;
		connectionRowPanel.add(connectButton, c);

		disconnectButton = new JButton("Disconnect");
		disconnectButton.addActionListener(this);
		disconnectButton.setMnemonic('d');
		c.gridx = 1;
		connectionRowPanel.add(disconnectButton, c);

		// Write row
		writeEntryLabel = new JLabel("Write: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.weightx = 0;
		c.ipady = 5;
		writeRowPanel.add(writeEntryLabel, c);

		writeEntry = new JTextField(15);
		writeEntry.setText(configWrite);
		c.gridx = 1;
		c.weightx = 0.5;
		writeRowPanel.add(writeEntry, c);

		// Communication row
		writeButton = new JButton("Write");
		writeButton.addActionListener(this);
		writeButton.setMnemonic('w');
		c.gridx = 0;
		commRowPanel.add(writeButton, c);

		readButton = new JButton("Read");
		readButton.addActionListener(this);
		readButton.setMnemonic('r');
		c.gridx = 1;
		commRowPanel.add(readButton, c);

		queryButton = new JButton("Query");
		queryButton.addActionListener(this);
		queryButton.setMnemonic('q');
		c.gridx = 2;
		commRowPanel.add(queryButton, c);

		// Read row
		readDisplayLabel = new JLabel("Read: ", SwingConstants.CENTER);
		c.gridx = 0;
		c.weightx = 0;
		readRowPanel.add(readDisplayLabel, c);

		readDisplay = new JTextField(15);
		c.gridx = 1;
		c.weightx = 0.5;
		readRowPanel.add(readDisplay, c);

		// Set up panel hierarchy
		mainPanel.add(hostRowPanel);
		mainPanel.add(portAddressRowPanel);
		mainPanel.add(connectionRowPanel);
		mainPanel.add(writeRowPanel);
		mainPanel.add(commRowPanel);
		mainPanel.add(readRowPanel);

		// Show the gui
		mainFrame.getContentPane().add(mainPanel, BorderLayout.CENTER);
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.pack();
		mainFrame.setLocationRelativeTo(null);
		updateGUIState();
		connectButton.requestFocus();
		mainFrame.setVisible(true);

	}

	// Event handler
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == connectButton) {
			connectButtonHandler();
		} else if (src == disconnectButton) {
			disconnectButtonHandler();
		} else if (src == writeButton) {
			writeButtonHandler();
		} else if (src == readButton) {
			readButtonHandler();
		} else if (src == queryButton) {
			queryButtonHandler();
		}
	}

	// Connect button handler
	private void connectButtonHandler() {
		int port;
		int address;
		isAttemptingToConnect = true;
		updateGUIState();
		try {
			port = Integer.parseInt(portEntry.getText());
			address = Integer.parseInt(addressEntry.getText());
			gpibThread.configure(hostEntry.getText(), port, address);
			gpibThread.connect(connectStatusHandler);
			readDisplay.setText("");
		} catch (NumberFormatException ex) {
			popupErrorMessage("Invalid entry: check port and GPIB address");
			isConnected = false;
			isAttemptingToConnect = false;
			updateGUIState();
		} catch (TrivialCommunicationException ex) {
			popupErrorMessage(ex.getMessage());
			isAttemptingToConnect = false;
			isConnected = false;
			updateGUIState();
		}
	}

	// Disconnect button handler
	private void disconnectButtonHandler() {
		isClosingConnection = true;
		updateGUIState();
		try {
			gpibThread.close(closeStatusHandler);
		} catch (TrivialCommunicationException ex) {
			popupErrorMessage(ex.getMessage());
		}
	}

	// Write button handler
	private void writeButtonHandler() {
		String cmd = writeEntry.getText().trim();
		if (!cmd.equals("")) {
			try {
				gpibThread.write(writeStatusHandler, cmd);
			} catch (TrivialCommunicationException ex) {
				popupErrorMessage(ex.getMessage());
			}
		}
	}

	// Read button handler
	private void readButtonHandler() {
		try {
			gpibThread.read(readStatusHandler);
		} catch (TrivialCommunicationException ex) {
			popupErrorMessage(ex.getMessage());
		}
	}

	// Query button handler
	private void queryButtonHandler() {
		String cmd = writeEntry.getText().trim();
		if (!cmd.equals("")) {
			try {
				gpibThread.query(readStatusHandler, cmd);
			} catch (TrivialCommunicationException ex) {
				popupErrorMessage(ex.getMessage());
			}
		}
	}

	// Update GUI state (enables and disables buttons)
	public void updateGUIState() {
		if (isConnected) {
			if (isClosingConnection) { 
				disconnectButton.setEnabled(false);
				writeButton.setEnabled(false);
				readButton.setEnabled(false);
				queryButton.setEnabled(false);
			} else {
				// Is connected
				connectButton.setEnabled(false);
				disconnectButton.setEnabled(true);
				writeButton.setEnabled(true);
				readButton.setEnabled(true);
				queryButton.setEnabled(true);
				hostEntry.setEnabled(false);
				portEntry.setEnabled(false);
				addressEntry.setEnabled(false);
			}
		} else {
			if (isAttemptingToConnect) {
				// Is attempting to connect
				connectButton.setEnabled(false);
				hostEntry.setEnabled(false);
				portEntry.setEnabled(false);
				addressEntry.setEnabled(false);
			} else {
				// Is not connected
				connectButton.setEnabled(true);
				disconnectButton.setEnabled(false);
				writeButton.setEnabled(false);
				readButton.setEnabled(false);
				queryButton.setEnabled(false);
				hostEntry.setEnabled(true);
				portEntry.setEnabled(true);
				addressEntry.setEnabled(true);
			}
		}
	}	

	// Thread-safe way to call updateGUIState()
	public void updateGUIStateSafe() {
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				updateGUIState();
			}
		} );
	}

	// Thread-safe way to set read field
	private void setReadField() {
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				readDisplay.setText(readField);
			}
		} );
	}

	// Show an error message
	public void popupErrorMessage(String message) {
		errorMessage = message;
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(mainFrame,errorMessage,
					"Error",JOptionPane.ERROR_MESSAGE);
			}
		} );
	}

	// Main program
	public static void main(String[] args) {
		GPIBInteractiveControl gpib_ic;
		gpib_ic = new GPIBInteractiveControl();
	}

	// Handler for a returned connect
	private class ConnectStatusHandler implements CommStatusHandler {
		public void commStatusHandler(CommStatus cs) {
			//System.out.println("inside ConnectStatusHandler");
			isConnected = cs.isConnected;
			if (cs.errorNum != 0) {
				if (cs.errorMessage != null) {
					popupErrorMessage(cs.errorMessage);
				} else {
					popupErrorMessage("Unknown error occurred on connect");
				}
			}
			isAttemptingToConnect = false;
			updateGUIStateSafe();
		}
	}

	// Handler for a returned close
	private class CloseStatusHandler implements CommStatusHandler {
		public void commStatusHandler(CommStatus cs) {
			//System.out.println("inside CloseStatusHandler");
			isConnected = cs.isConnected;
			if (cs.errorNum != 0) {
				if (cs.errorMessage != null) {
					popupErrorMessage(cs.errorMessage);
				} else {
					popupErrorMessage("Unknown error occurred on close");
				}
			}
			isClosingConnection = false;
			updateGUIStateSafe();
		}
	}

	// Handler for a returned write
	private class WriteStatusHandler implements CommStatusHandler {
		public void commStatusHandler(CommStatus cs) {
			boolean oldIsConnected = isConnected;
			//System.out.println("inside WriteStatusHandler");
			isConnected = cs.isConnected;
			if (cs.errorNum != 0) {
				if (cs.errorMessage != null) {
					popupErrorMessage(cs.errorMessage);
				} else {
					popupErrorMessage("Unknown error occurred on write");
				}
			}
			if (isConnected != oldIsConnected) {
				updateGUIStateSafe();
			}
		}
	}

	// Handler for a returned read or query
	private class ReadStatusHandler implements CommStatusHandler {
		public void commStatusHandler(CommStatus cs) {
			boolean oldIsConnected = isConnected;
			//System.out.println("inside ReadStatusHandler");
			isConnected = cs.isConnected;
			if (cs.errorNum != 0) {
				if (cs.errorMessage != null) {
					popupErrorMessage(cs.errorMessage);
				} else {
					popupErrorMessage("Unknown error occurred on read or query");
				}
			}
			readField = cs.readMessage;
			setReadField();
			if (isConnected != oldIsConnected) {
				updateGUIStateSafe();
			}
		}
	}

}
